#ifndef XSIL_DIM_HH
#define XSIL_DIM_HH

#include "xsil/xobj.hh"

namespace xsil {
    class Xwriter;
    class xobj;
    class dim;

/**  xsil::dim carries the XSIL Array object's dimension information.
  *  @memo Array dimension class.
  *  @author J. Zweizig
  *  @version 1.1; Modified December 13, 1999
  */
class dim : public xobj {
public:
    /** Default constructor.
     */
    dim(void);

    /**  Construct an object with a non-zero dimension size and an
      *  optional name. If the name is coded as a null pointer, the 
      *  name attribute is left blank.
      *  @memo Full dimension constructor.
      */
    dim(const char* Name, int N);

    /**  Construct a dimension object identical to the argument object.
      *  @memo Dimension copy constructor.
      *  @param Dim Dimension object to be copied.
      */
    dim(const dim& Dim);

    /**  Destructor.
      */
    ~dim(void);

    /**  The dimension object is written to an XML output stream specified
      *  by the argument object. The calling sequesnce is kept compatible 
      *  with the xobj abstract class in case it is decided that dim should 
      *  inherit xobj;
      *  @memo Write out the dimension object to an XML file.
      *  @param xout XML writer to which the dimension is to be written.
      */
    void Spew(Xwriter& xout) const;

    /**  Clone a dimension object.
      *  @memo Get object type.
      *  @return Object type;
      */
    xobj* Clone(void) const;

    /**  Returns a charater string "Dim" representing the object type.
      *  @memo Get object type.
      *  @return Object type;
      */
    const char* getObjType(void) const;

    /**  Get the dimension length.
      *  Returns aninteger length for this dimension.
      */
    int  getDim(void) const;

  /** Set the dimension.
    *  Set the dimension length to the integer argument value. 
    */
    void setDim(int Dim);

private:
    int    mDim;
};

}
#endif  //  XSIL_DIM_HH
